% kv_read_halpern Read data in UC format
%
%   data = d01read(filename,...)
%   [ax,data] = d01read(filename,...);
%   [ax,data,dsc] = d01read(filename,...);

function varargout=kv_read_halpern(filename, varargin)

opt = [];
if nargin > 1,
    if ~mod(nargin-1,2)
        for kk=1:2:nargin-1
            opt=setfield(opt, lower(varargin{kk}), varargin{kk+1});    
        end
    else, error('Wrong amount of arguments')
    end
else, error('Usage: kv_read_halpern(filename, src). Options: is2D.');
end
[pathstr, fname, ext] = fileparts(filename);

switch lower(ext)
    case {'.img'}
        [mat, mat_info] = read_img_file(fullfile(pathstr,fname));
        sz = size(mat);
        spec = reshape(mat, [sz(1), prod(sz(2:end))]);
        ax.x = (1:sz(1))';
        ax.y = (1:prod(sz(2:end)))';
    otherwise
        % type *.number 
        slice = str2num(ext(2:end));
        if isempty(slice)
            % type *.Snumber 
            slice = str2num(ext(3:end));
        end            
        [pars_out,Z,range_out,prefix,title,comment]=...
            read_bin_spectrum(fullfile(pathstr,fname),slice,slice,1);
        spec = Z(:,2);
        ax.x = Z(:,1);
        ax.xlabel = 'Field,G';
end
dsc = [];

% assign output depending on number of output arguments
% requested
switch nargout
 case 1,
   varargout = {spec};
 case 2,
   varargout = {ax, spec};
 case 3,
   varargout = {ax, spec, dsc};
end